# This Python file uses the following encoding: utf-8
from typing import List
from imdb import Cinemagoer
from dataclasses import dataclass

@dataclass
class MovieData():
    '''Dataclass used to store all data related to a movie'''
    id: str
    title: str
    year: str
    cover_url: str
    rating: str
    summary: str = ""
    plot: str = ""
    trailer_url: str = ""
    fully_searched: bool = False

class ImdbSearcher():
    '''
    Interacts with the IMDbPY API and maintain a list of MovieData for holding current search result
    The intended use is to :
    1) Make a title search which returns a fast result with movie titles but not much details
    2) Make a search by ID for a specific movie with detailed informations
    '''
    def __init__(self) -> None:
        self.imdbApi = Cinemagoer()
        self.moviesList: List[MovieData] = []

    def searchByTitle(self, title, max_result = 10, no_series = False):
        print("Search movie by title: {}".format(title))
        print("Include TV : %d" % (not no_series))
        try:
            # TODO: search_movie_advanced() does not work anymore ?
            movieResult = self.imdbApi.search_movie(title, results=max_result)
        except Exception:
            print("No results")
            return None
        if movieResult is None:
            print("No results")
            return None
        else:
            for i in range(len(movieResult)):
                cover_url = None
                if 'full-size cover url' in movieResult[i].keys():
                    cover_url = movieResult[i]['full-size cover url']
                if no_series and ('kind' in movieResult[i].keys()) and movieResult[i]['kind'].lower() != "movie":
                    print("skipped %s serie" % movieResult[i]['title'])
                    # Skip series
                    continue
                id = movieResult[i].getID()
                year = self.findMovieInfo(movieResult[i], 'year')
                rating = self.findMovieInfo(movieResult[i], 'rating')
                mov = MovieData(id, movieResult[i]['long imdb title'], year, cover_url, rating)
                self.moviesList.append(mov)
        return self.moviesList
        
    def searchById(self, id):
        print("Search movie by ID: {}".format(id))
        movieResult = self.imdbApi.get_movie(id)
        mov = self.getMovieFromId(id)
        vids = self.findMovieInfo(movieResult, 'videos')
        if vids:
            trailer_url = "https://www.imdb.com/video/imdb/"+vids[0].rsplit('/', 1)[-1]+ "/imdb/embed?autoplay=false&width=720"
        if not mov:
            
            movieObj = MovieData(id, 
                    self.findMovieInfo(movieResult, 'long imdb title'), 
                    self.findMovieInfo(movieResult, 'year'), 
                    self.findMovieInfo(movieResult, 'full-size cover url'), 
                    self.findMovieInfo(movieResult, 'rating'), 
                    movieResult.summary(),
                    self.findMovieInfo(movieResult, 'plot outline'),
                    trailer_url if vids else None,
                    fully_searched=True)
            self.moviesList.append(movieObj)
            return movieObj
        else:
            mov.fully_searched = True
            mov.rating = self.findMovieInfo(movieResult, 'rating')
            mov.year = self.findMovieInfo(movieResult, 'year')
            mov.summary = movieResult.summary()
            mov.plot = self.findMovieInfo(movieResult.data, 'plot outline')
            return mov

    def getCoverUrl(self, id):
        # getting cover url of the series
        return self.imdbApi.get_movie(id).data['full-size cover url']

    def getSummary(self, id):
        return self.imdbApi.get_movie(id).summary()

    def getMovieFromTitle(self, title:str):
        '''
        Find a movie in the local movieResult object
        '''
        for mov in self.moviesList:
            if mov.title == title:
                return mov
        return None

    def getMovieFromId(self, id:str):
        '''
        Find a movie in the local movieResult object
        '''
        for mov in self.moviesList:
            if mov.id == id:
                return mov
        return None

    def findMovieInfo(self, movieResult, key:str):
        '''
        Find movie info from the Movie object, if available
        '''
        if key in movieResult.keys():
            return movieResult[key]
        return None
    
    def clear(self):
        print("Clear IMDb search results !")
        self.moviesList = []



# This Python file uses the following encoding: utf-8
from typing import List

from pyimdbmoviefinder.YtsFetcher import YtsFetcher
from pyimdbmoviefinder.JackettFetcher import JackettFetcher
from pyimdbmoviefinder.TorrentFetcher import TorrentResult
from dataclasses import dataclass

@dataclass
class TorrentData():
    id: str
    torrents : List[TorrentResult]

class TorrentSearcher:
    def __init__(self):
        self.fetchers = []
        self.torrentsList: List[TorrentData] = []

    def setSearch(self, id:str, title:str, yts:bool=True, jackett:bool=True, jackettApiKey:str=None, jackettHost:str=None):
        self.id = id
        if yts:
            self.fetchers.append(YtsFetcher(id))
        if jackett:
            assert jackettApiKey, "Set a valid API key to use jackett indexers"
            self.fetchers.append(JackettFetcher(id, title, jackettApiKey, jackettHost))

    def run(self):
        result = []
        for fetcher in self.fetchers:
            tr = fetcher.fetch()
            if tr is not None:
                result += tr
        newTorrents = TorrentData(self.id, result)
        existing = self.getTorrentsDataFromId(self.id)
        if existing:
            existing.torrents = result
        else:
            self.torrentsList.append(newTorrents)
        self.fetchers = []
        return newTorrents

    def getTorrentsDataFromId(self, id):
        for data in self.torrentsList:
            if data.id == id:
                return data
        return None
    
    def getTorrentsFromId(self, id):
        for data in self.torrentsList:
            if data.id == id:
                return data.torrents
        return None
    
    def clear(self):
        self.torrentsList = []



from typing import List
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
from pyimdbmoviefinder.TorrentFetcher import TorrentFetcher, TorrentResult

class YtsFetcher(TorrentFetcher):
    def __init__(self, id):
        self.url = "https://yts.mx/api/v2/list_movies.json?query_term="
        self.movie_id = "tt"+id

    def requests_retry_session(
            self,
            retries=3,
            backoff_factor=0.3,
            status_forcelist=(500, 502, 504),
            session=None,
        ):
            session = session or requests.Session()
            retry = Retry(
                total=retries,
                read=retries,
                connect=retries,
                backoff_factor=backoff_factor,
                status_forcelist=status_forcelist,
            )
            adapter = HTTPAdapter(max_retries=retry)
            session.mount('http://', adapter)
            session.mount('https://', adapter)
            return session

    def fetch(self) -> List[TorrentResult]:
        api_url = self.url + self.movie_id
        response = self.requests_retry_session().get(api_url, timeout=120).json()
        #print(response)
        data = response.get('data')
        movies = data.get('movies')
        if movies is None:
            print("Torrents not found on YTS")
            return None
        for movie in movies:
            title_long = movie.get('title_long')
            print("Found torrents on YTS for :", title_long)
            torrents = movie.get('torrents')
            if torrents is None:
                print("no torrent for this movie")
                continue
            descs = []
            for torrent in torrents:
                #print(torrent)
                desc = TorrentResult(title_long, 
                                    torrent.get('quality'), 
                                    torrent.get('type'), 
                                    torrent.get('seeds'), 
                                    torrent.get('size'),
                                    "YTS", 
                                    torrent.get('url'))
                descs.append(desc)
        return descs



import argparse
import configparser
import pathlib
import sys
from getpass import getpass
import threading
import time
from pyimdbmoviefinder import ImdbSearcher
from pyimdbmoviefinder import TorrentSearcher
from pyimdbmoviefinder import TorrentDownloader

DEFAULT_MAX_RESULT = 8

class Spinner:
    busy = False
    delay = 0.1

    @staticmethod
    def spinning_cursor():
        while 1: 
            for cursor in '|/-\\': yield cursor

    def __init__(self, delay=None):
        self.spinner_generator = self.spinning_cursor()
        if delay and float(delay): self.delay = delay

    def spinner_task(self):
        while self.busy:
            sys.stdout.write(next(self.spinner_generator))
            sys.stdout.flush()
            time.sleep(self.delay)
            sys.stdout.write('\b')
            sys.stdout.flush()

    def __enter__(self):
        self.busy = True
        threading.Thread(target=self.spinner_task).start()

    def __exit__(self, exception, value, tb):
        self.busy = False
        time.sleep(self.delay)
        if exception is not None:
            return False
        
def showImdb(imdbResult):
    for i, movie in enumerate(imdbResult):
        s_link = "https://www.imdb.com/title/tt" + movie.id
        print(str(i) + ": " + movie.title + s_link)
    # Ask user choice
    try:
        num = int(input("Enter choice: "))
    except ValueError:
        print("Invalid input")
        sys.exit(0)
    assert num >= 0 and num < len(imdbResult), "Invalid input"
    for i, movie in enumerate(imdbResult):
        if i == num:
            return movie
    return None

def showTorrent(torrentResult):
    for i, torrent in enumerate(torrentResult.torrents):
        s = str(i) + ": "
        s += torrent.title
        s += f" ({torrent.quality})"
        s += f" ({torrent.provider})"
        s += f' seeders :{torrent.seeds}'
        print(s)
    # Ask user choice
    try:
        num = int(input("Enter choice: "))
    except ValueError:
        print("Invalid input")
        sys.exit(0)
    assert num >= 0 and num < len(torrentResult.torrents), "Invalid input"
    for i, torrent in enumerate(torrentResult.torrents):
        if i == num:
            return torrent
    return None

def cli():                
    print('PyTorrSearch CLI')
    parser = argparse.ArgumentParser(description="PyTorrSearch CLI usage")
    parser.add_argument("-t","--title", help="Search movie by Title")
    parser.add_argument("-i","--id", help="Search movie by ID")
    parser.add_argument("-a","--all", help="Search torrents on all providers, otherwise only YTS is used", action="store_true")
    parser.add_argument("-n","--num", help="Maximum number of search results")
    if len(sys.argv)==0:
        parser.print_help()
        parser.exit()
    args = vars(parser.parse_args())
    print(args)
    
    max_result = args["num"] if args["num"] else DEFAULT_MAX_RESULT
    all_providers = args["all"]
    
    # User Configuration 
    config = configparser.ConfigParser()
    config_path = str(pathlib.Path(__file__).parent.parent.parent) + "/config/config.ini"
    config.read(config_path)
    if config.has_section("Jackett"):
        jackettHost = config.get("Jackett", "Host")
        jackettApiKey = config.get("Jackett", "ApiKey")
        
    # 1. Search IMDb
    with Spinner():
        if args["title"]:
            imdbResult = ImdbSearcher().searchByTitle(args['title'], max_result, no_series = True)
        elif args["id"]:
            imdbResult = ImdbSearcher().searchById(args['id'])
        else:
            parser.print_help()
            parser.exit()

    if imdbResult is None:
        print("No IMDb results")
        sys.exit(0)
    else:
        choice = showImdb(imdbResult)

    # Search torrents
    with Spinner():
        searcher = TorrentSearcher()
        searcher.setSearch(choice.id, choice.title, yts=True, jackett=all_providers,  jackettApiKey=jackettApiKey, jackettHost=jackettHost)
        torrentResult = searcher.run()
    
    if torrentResult is None:
        print("No Torrents found")
        sys.exit(0)
    else:
        choice = showTorrent(torrentResult)

    # RPC server configuration
    if config.has_section("RPC"):
        print("Using config.ini for RPC configuration")
        rpc_host, rpc_user, rpc_password = \
            config.get('RPC','Host'), config.get('RPC','User'), config.get('RPC','Password')
    else:
        print("No config.ini found, enter RPC config manually:")
        rpc_host = input('RPC Address: ')
        rpc_user = input('Username: ')
        rpc_password = getpass()

    # Send to server
    dl = TorrentDownloader(rpc_host, rpc_user, rpc_password)
    print(dl.add_torrent_magnet(choice.url) + " to " + rpc_host)

if __name__ == '__main__':
    cli()



import operator
from typing import List

from requests import HTTPError
from pyimdbmoviefinder.TorrentFetcher import TorrentFetcher, TorrentResult
import humanize

import logging
import re

import xml.etree.ElementTree as ET

from pyimdbmoviefinder.http_utils import build_url, fetch_url

logger = logging.getLogger('torrentSearch')
DEFAULT_HOST = "http://localhost:9117"

class JackettFetcher(TorrentFetcher):
    """docstring for JackettFetcher"""
    def __init__(self, id, title, apiKey, host=DEFAULT_HOST, path="torznab/all", limit=25, ssl=False):
        self.movie_id = "tt"+id
        self.title = title
        if not host:
            host = DEFAULT_HOST
            
        print(f'Host {host}, API key {apiKey}')
        self.api = Jackett(apiKey, host, path, limit, ssl)
    
    def fetch(self) -> List[TorrentResult]:
        torrents = self.api.search(self.title)
        if not torrents:
            # Give up
            return None
        torrents_final = []
        for torrent in torrents:
            if 'chinese' in torrent.title.lower():
                # Lots of chinese ensubbed movies on rarbg...
                continue
            if torrent.seeds is not None and torrent.seeds == 0:
                print("No seeders, skipping...")
                continue
            print("Found torrent with seeders:", torrent.title)
            torrents_final.append(torrent)
        # Sort by seeders
        sorted_torrents = sorted(torrents_final, key= lambda e:e.seeds, reverse=True)
        return sorted_torrents

class Jackett(object):
    """docstring for Jackett"""
    def __init__(self, apikey, host, path, limit, ssl):
        super(Jackett, self).__init__()
        self.apikey = apikey
        self.host = host
        self.path = path
        self.page_limit = limit
        self.ssl = ssl

    def get_apikey(self):
        logger.debug('Using api key: {}'.format(self.apikey))
        return self.apikey

    def get_path(self):
        return self.path

    def get_page_limit(self):
        logger.debug('Current page limit: {} pages'.format(self.page_limit))
        return self.page_limit

    def search(self, query):
        """
        Starts the call to getting result from our indexer
        :param jackett.Jackett self: object instance
        :param str query: query we want to search for
        :return: list of results we found from scraping jackett output based on query
        :rtype: list
        """
        path = self.get_path().split('/')
        url_args = {
            'apikey': self.get_apikey(),
            'limit': self.get_page_limit(),
            'q': query
        }
        logger.debug('Url arguments for jackett search: {}'.format(url_args))

        url = build_url(self.ssl, self.host, path, url_args)
        url = url.replace('+', '%20')
        res = fetch_url(url)

        return self.parse_xml_for_torrents(res.read())


    def find_xml_attribute(self, xml_element, attr):
        """
        Finds a specific XML attribute given a element name
        :param jackett.Jackett self: object instance
        :param xml.etree.ElementTree.Element xml_element: the xml tree we want to search
        :param str attr: the attribute/element name we want to find in the xml tree
        :return: the value of the element fiven the attr/element name
        :rtype: str
        """
        value = xml_element.find(attr)
        if (value != None):
            logger.debug('Found attribute: {}'.format(attr))
            return value.text
        else:
            logger.warning('Could not find attribute: {}'.format(attr))
            return ''

    def parse_xml_for_torrents(self, raw_xml):
        """
        Finds a specific XML attribute given a element name
        :param jackett.Jackett self: object instance
        :param bytes raw_xml: the xml page returned by querying jackett
        :return: all the torrents we found in the xml page
        :rtype: list
        """
        tree = ET.fromstring(raw_xml)
        channel = tree.find('channel')
        results = []
        
        for child in channel.findall('item'):
            title = self.find_xml_attribute(child, 'title')
            date = self.find_xml_attribute(child, 'pubDate')
            magnet = self.find_xml_attribute(child, 'link')
            size = self.find_xml_attribute(child, 'size')
            indexer = self.find_xml_attribute(child, 'jackettindexer')
            foundUploader = re.findall('-1? *\w*', title)
            if len(foundUploader) > 0:
                uploader = str(foundUploader[-1][1:])
            else:
                uploader = ''
            seeders = 0
            peers = 0

            for elm in child.findall('{http://torznab.com/schemas/2015/feed}attr'):
                if elm.get('name') == 'seeders':
                    seeders = elm.get('value')
                if elm.get('name') == 'peers':
                    peers = elm.get('value')
                try:
                    size = int(size)
                    size = humanize.naturalsize(size)
                except ValueError:
                    pass

            logger.debug('Found torrent with info: \n\ttitle: {}\n\tmagnet: {}\n\tsize: {}\n\tdate: {}\
            \n\tseeders: {}\n\tpeers: {}'.format(title, magnet, size, date, seeders, peers))
            torrent = TorrentResult(title, 
                                    '?',
                                    '?', 
                                    seeders, 
                                    size,
                                    f'{indexer} - {uploader}', 
                                    magnet)
            torrent.quality = torrent.find_release_type()
            # Let's just skip cam torrent...
            if 'cam' in torrent.quality:
                continue
            results.append(torrent)

        return results


# This Python file uses the following encoding: utf-8
from abc import abstractmethod
from dataclasses import dataclass
from typing import List

RELEASE_TYPES = ('bdremux', 'brremux', 'remux',
	'bdrip', 'brrip', 'blu-ray', 'bluray', 'bdmv', 'bdr', 'bd5',
	'web-cap', 'webcap', 'web cap',
	'webrip', 'web rip', 'web-rip', 'web',
	'webdl', 'web dl', 'web-dl', 'hdrip',
	'dsr', 'dsrip', 'satrip', 'dthrip', 'dvbrip', 'hdtv', 'pdtv', 'tvrip', 'hdtvrip',
	'dvdr', 'dvd-full', 'full-rip', 'iso',
	'hdts', 'hdts', 'telesync', 'pdvd', 'predvdrip',
	'camrip', 'cam', '720p', '1080p', '2160p')

@dataclass
class TorrentResult():
    title: str
    quality: str
    type: str
    seeds : int
    size : str
    provider : str
    url : str
    description : str = ""

    def __post_init__(self):
        self.description = self.title + " " + str(self.find_release_type())
        
    def find_release_type(self):
        name = self.title.casefold()
        return [r_type for r_type in RELEASE_TYPES if r_type in name]

class TorrentFetcher:
    def __init__(self):
        pass

    @abstractmethod
    def fetch(self) -> List[TorrentResult]:
        pass




# This Python file uses the following encoding: utf-8
import httplib2
from json import dumps

class TorrentDownloader:
    '''
    Class allowing torrent download to a transmission daemon
    '''
    def __init__(self, host, user, pw, dir=None):
        self.host = host
        self.user = user
        self.pw = pw
        self.dir = dir

    def add_torrent_magnet(self, magnet_link:str) -> str:
        '''
        Forward the magnet link to transmission daemon through RPC
        magnet_link : the magnet link of the torrent to be sent
        return: String describing success / failure
        '''
        h = httplib2.Http(".cache")
        h.add_credentials(self.user, self.pw)
        try:
            resp, content = h.request(self.host, "GET")
        except Exception as e:
            return "Unable to send the request, verify your config : \r\n {}".format(str(e))
        if not 'x-transmission-session-id' in resp.keys():
            return "Response missing x-transmission-session-id, check your hostname/user/password or webserver configuration !"
        headers = { "X-Transmission-Session-Id": resp['x-transmission-session-id'] }
        args = { "filename": magnet_link }
        if self.dir:
            args['download-dir'] = self.dir
        body = dumps( { "method": "torrent-add",
                        "arguments": args } )
        _, content = h.request(self.host, 'POST', headers=headers, body=body)

        if str(content).find("success") == -1:
            print("ERROR: Magnet Link: " + magnet_link)
            print("ERROR: Answer: " + str(content))
            return "An error occured while sending torrent to {}. Server answered \r\n\"{}\"\
                \r\nMake sure the link is correct".format(self.host, str(content))
        return "Successfully added {}".format(magnet_link)



from setuptools import setup, find_packages

setup(name='pyimdbmoviefinder', version='1.0', packages=find_packages(), entry_points={
        'console_scripts': ['pyimdbmoviefinder=pyimdbmoviefinder.cli.clisearch:cli'] # so this directly refers to a function available in __init__.py
        })